#!/usr/bin/python3

from subprocess import call
import sys
import getopt
sys.path.append('/usr/lib/mintstick')
from mountutils import do_umount
import syslog
import os

def name_has_digit(name):
        for i in name:
            if i.isdigit():
                return True

        return False

def call_fsync(path):
    f = open(path, 'r')
    os.fsync(f)
    f.close()

def execute(command, path):
    syslog.syslog(str(command))
    call(command)
    call_fsync(path)

def raw_format(path, fstype, volume_label, uid, gid):

    partition_path = path

    do_umount(path)

    if fstype == "fat32":
        partition_type = "fat32"
    elif fstype == "exfat":
        partition_type = "ntfs"
    elif fstype == "ntfs":
        partition_type = "ntfs"
    elif fstype == "ext4":
        partition_type = "ext4"

    # First erase MBR and partition table , if any
    execute(["dd", "if=/dev/zero", "of=%s" % path, "bs=1M", "count=1"], path) # 32K

    if not name_has_digit(path):
        # Make the partition table
        execute(["parted", path, "mktable", "msdos"], path)

        # Make a partition (primary, with FS ID ext3, starting at 1MB & using 100% of space).
        # If it starts at 0% or 0MB, it's not aligned to MB's and complains
        execute(["parted", path, "mkpart", "primary", partition_type, "1", "100%"], path)

        partition_path = "%s1" % path

    # Call wipefs on the new partitions to avoid problems with old filesystem signatures
    execute(["wipefs", "-a", partition_path, "--force"], partition_path)

    # Format the FS on the partition
    if fstype == "fat32":
        execute(["mkdosfs", "-F", "32", "-n", volume_label, partition_path], partition_path)
    elif fstype == "exfat":
        execute(["mkfs.exfat", "-n", volume_label, partition_path], partition_path)
    elif fstype == "ntfs":
        execute(["mkntfs", "-f", "-L", volume_label, partition_path], partition_path)
    elif fstype == "ext4":
        execute(["mkfs.ext4", "-E", "root_owner=%s:%s" % (uid, gid), "-L", volume_label, partition_path], partition_path)

    call_fsync(partition_path)
    device_path = ''.join([i for i in path if not i.isdigit()])
    call_fsync(device_path)

    # Exit
    sys.exit(0)

def main():
    # parse command line options
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:f:l:u:g:", ["help", "device=","filesystem=","label=","uid=","gid="])
    except getopt.error as msg:
        print(msg)
        print("for help use --help")
        sys.exit(2)

    for o, a in opts:
        if o in ("-h", "--help"):
            print("Usage: %s -d device -f filesystem -l volume_label\n"  % sys.argv[0])
            print("-d|--device          : device path")
            print("-f|--filesystem      : filesystem\n")
            print("-l|--label           : volume label\n")
            print("-u|--uid             : uid of user\n")
            print("-g|--gid             : gid of user\n")
            print("Example : %s -d /dev/sdj -f fat32 -l \"USB Stick\" -u 1000 -g 1000" % sys.argv[0])
            sys.exit(0)
        elif o == "-d":
            device = a
        elif o == "-f":
            if a not in [ "fat32", "exfat", "ntfs", "ext4" ]:
                print("Specify fat32, exfat, ntfs or ext4")
                sys.exit(3)
            fstype = a
        elif o == "-l":
            label = a
        elif o == "-u":
            uid = a
        elif o in "-g":
            gid = a

    argc = len(sys.argv)
    if argc < 11:
      print("Too few arguments")
      print("for help use --help")
      exit(2)

    raw_format(device, fstype, label, uid, gid)

if __name__ == "__main__":
    main()
